package com.persagy.dmp.rwd.migrate.controller;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.StrUtil;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.common.utils.ResultHelper;
import com.persagy.dmp.rwd.migrate.service.IDigitalMigrateService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 版本定义Controller
 * @author Charlie Yu
 * @date 2021-06-23
 */
@RestController
@RequestMapping("/migrate/digital")
public class DigitalMigrateController {

    @Autowired
    private IDigitalMigrateService service;

    /**
     * 启动数据迁移
     * 迁移范围：自定义数据字典、对象、关系
     * @param requestMap
     * {
     *     "orgUrl":"数据中台组织服务url前缀（不含最后的/)",
     *     "oldSchema":"数据中台使用的数据库实例",
     *     "groupCode":"指定集团编码【单个或数组】 - 为空表示不限集团",
     *     "projectId":"指定项目ID【单个或数组】 - 为空表示不限项目"
     * }
     */
    @PostMapping("/start")
    public CommonResult<String> query(@RequestBody Map<String, Object> requestMap) {
        // 处理参数
        final String orgUrl = MapUtil.getStr(requestMap, "orgUrl");
        final String oldSchema = MapUtil.getStr(requestMap, "oldSchema");
        if(StrUtil.isBlank(orgUrl) || StrUtil.isBlank(oldSchema) ) {
            throw new BusinessException(ResponseCode.A0400.getCode(), ResponseCode.A0400.getDesc());
        }
        // 集团编码
        final List<String> groupCodes = new ArrayList<>();
        Object groupCode = requestMap.get("groupCode");
        if(groupCode != null) {
            if(groupCode instanceof String) {
                groupCodes.add((String) groupCode);
            } else if(groupCode instanceof List) {
                groupCodes.addAll((List<String>) groupCode);
            }
        }
        // 项目ID
        final List<String> projectIds = new ArrayList<>();
        Object projectId = requestMap.get("projectId");
        if(projectId != null) {
            if(projectId instanceof String) {
                projectIds.add((String) projectId);
            } else if(projectId instanceof List) {
                projectIds.addAll((List<String>) projectId);
            }
        }
        // 启动迁移 执行时间较长，后台线程运行 TODO 需要加分布式锁，并支持查看执行情况
        ThreadUtil.execute(() -> service.startMigrate(groupCodes, projectIds, orgUrl, oldSchema));
        return ResultHelper.success();
    }
}
